#include <QDBusConnection>
#include <QDBusInterface>
#include <QDBusReply>

#include "Monitor.h"
#include "GenerateLog.h"
#include "Settings.h"
#include "Bubble.h"
#include "FirstRunWizard.h"
#include "HelpDialog.h"

#include "Applet.h"

const int MIN_APPLET_VERSION = 100241;
const int APPLET_VERSION = 100277;

const QString UBUDSL_PREFIX( QString( PKG_PREFIX ).remove( "\"" ) );

const QString Applet::desktopFileName = "ubudsl_applet.desktop";
const QString Applet::autostartPath = QDir::homePath() + "/.config/autostart";

QDBusInterface daemon_iface( "com.ubudsl.Daemon", "/com/ubudsl/Daemon" , "com.ubudsl.Daemon", QDBusConnection::systemBus() );

Applet::Applet( QWidget *parent )
        : QSystemTrayIcon( parent ),
        m_settings( "UbuDSL", "UbuDSL Applet" ),
        m_daemonOk( false ),
        m_menu( new QMenu() ),
        iconConnected( ":/images/connected.png" ),
        iconConnecting( ":/images/connecting.png" ),
        iconDisconnected( ":/images/disconnected.png" ),
        iconConnect( ":/images/network-connect.png" ),
        iconDisconnect( ":/images/network-disconnect.png" ),
        iconReconnect( ":/images/network-reconnect.png" ),
        iconHide( ":/images/hide.png" ),
        iconNoDaemon( ":/images/nodaemon.png" ),
        iconNoModem( ":/images/nomodem.png" ),
        iconShow( ":/images/show.png" ),
        m_newModemMsgBox( new QMessageBox( 0 ) ),
        m_connectingTimer( new QTimer() ),
        m_timer( new QTimer() ),
        m_monitor( new Monitor() ),
        m_generateLog( new GenerateLog( this ) ),
        m_settingsDialog( new Settings() ),
        m_bubble( new Bubble() ),
        m_helpDialog( new HelpDialog() )
{
    m_actionHide      = m_menu->addAction( iconShow, tr( "&Show" ) );
    m_actionConnect   = m_menu->addAction( iconConnect, tr( "&Connect" ) );
    m_actionReconnect = m_menu->addAction( iconReconnect, tr( "&Reconnect" ) );

    m_separator1 = m_menu->addSeparator();

    m_actionGenerateLog  = m_menu->addAction( QIcon( ":/images/log.png" ), tr( "&Generate log" ) );
    m_actionRestartModem = m_menu->addAction( QIcon( ":/images/restart.png" ), tr( "&Restart modem" ) );

    m_pluginsMenu = m_menu->addMenu( QIcon( ":images/plugins_menu.png" ), tr( "Plugins" ) );

    m_actionSettings     = m_menu->addAction( QIcon( ":/images/settings.png" ), tr( "&Settings" ) );
    
    m_separator2 = m_menu->addSeparator();

    m_actionAutostart = m_menu->addAction( tr( "Autostart Applet" ) );
    m_actionQuit      = m_menu->addAction( QIcon( ":/images/quit.png" ), tr( "&Quit" ) );


    m_actionConnect->setEnabled( false );
    m_actionReconnect->setEnabled( false );
    m_monitor->setConnectEnabled( false );
    m_actionAutostart->setCheckable( true );
    m_actionAutostart->setChecked( m_settings.value( "autostart", false ).toBool() );

    connect( m_monitor, SIGNAL( showHide() ), this, SLOT( showHideMonitor() ) );
    connect( m_monitor, SIGNAL( enableConnectButton( bool ) ), m_actionConnect, SLOT( setEnabled( bool ) ) );
    connect( m_actionConnect, SIGNAL( triggered() ), m_monitor, SLOT( connectDisconnect() ) );
    connect( m_actionReconnect, SIGNAL( triggered() ), m_monitor, SLOT( reconnect() ) );
    connect( m_actionHide, SIGNAL( triggered() ), this, SLOT( showHideMonitor() ) );
    connect( m_actionQuit, SIGNAL( triggered() ), this, SIGNAL( quit() ) );
    connect( m_actionGenerateLog, SIGNAL( triggered() ), m_generateLog, SLOT( start() ) );
    connect( m_actionSettings, SIGNAL( triggered() ), this, SLOT( showSettings() ) );
    connect( m_actionAutostart, SIGNAL( triggered() ), this, SLOT( autostartActionTriggered() ) );
    connect( m_actionRestartModem, SIGNAL( triggered() ), this, SLOT( restartModem() ) );
    connect( m_generateLog, SIGNAL( started() ), this, SLOT( disableGenerateLog() ) );
    connect( m_generateLog, SIGNAL( finished() ), this, SLOT( enableGenerateLog() ) );
    connect( m_timer, SIGNAL( timeout() ), this, SLOT( checkDaemonConnection() ) );
    connect( m_connectingTimer, SIGNAL( timeout() ), this, SLOT( connecting() ) );
    connect( m_settingsDialog->ui.buttonSave, SIGNAL( clicked() ), this, SLOT( applySettings() ) );
    connect( this, SIGNAL( activated( QSystemTrayIcon::ActivationReason ) ),
             this, SLOT( clicked( QSystemTrayIcon::ActivationReason ) ) );

    setContextMenu( m_menu );

    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "connectionEstablished",
                                          this, SLOT( connectionEstablished() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "connectionTerminated",
                                          this, SLOT( connectionTerminated( const bool ) ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "establishingConnection",
                                          this, SLOT( establishingConnection() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "restartModemStarted",
                                          this, SLOT( restartModemStarted() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "restartModemFinished",
                                          this, SLOT( restartModemFinished() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "connectionTimeout",
                                          this, SLOT( connectionTimeout() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "settingOpenDNS",
                                          this, SLOT( settingOpenDNS( const bool ) ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "incorrectDNS",
                                          this, SLOT( incorrectDNS() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "modemUnplugged",
                                          this, SLOT( modemUnplugged() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "newModemUnplugged",
                                          this, SLOT( newModemUnplugged() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "modemPlugged",
                                          this, SLOT( modemPlugged( const QString & ) ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "newModemPlugged",
                                          this, SLOT( newModemPlugged( const QString & ) ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "accountDataChanged",
                                          this, SLOT( accountDataChanged( const bool ) ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "compilationFinished",
                                          this, SLOT( compilationFinished( const bool ) ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "compilingDriver",
                                          this, SLOT( compilingDriver() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "connectionTimeout",
                                          this, SIGNAL( playSndError() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "synchronising",
                                          this, SLOT( synchronising() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "synchronisationFailed",
                                          this, SLOT( synchronisationFailed() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "invalidAccountData",
                                          this, SLOT( invalidAccountData() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "restartOnModemHangUp",
                                          this, SLOT( restartOnModemHangUp() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "PADOTimeout",
                                          this, SLOT( PADOTimeout() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "modemStillHangsUp",
                                          this, SLOT( modemStillHangsUp() ) );
    QDBusConnection::systemBus().connect( "", "", "com.ubudsl.Daemon", "connectionState",
                                          this, SLOT( setConnectionState( const int & ) ) );

    m_newModemMsgBox->setAttribute( Qt::WA_QuitOnClose, false );
    m_newModemMsgBox->setIcon( QMessageBox::Question );
    m_newModemMsgBox->addButton( tr( "&No" ), QMessageBox::NoRole );
    m_btnYes = m_newModemMsgBox->addButton( tr( "&Yes" ), QMessageBox::YesRole );

    QRect d( QApplication::desktop()->screenGeometry() );
    m_monitor->move(( d.width() - m_monitor->width() ) / 2, ( d.height() - m_monitor->height() ) / 2 );

    m_timer->setInterval( 2000 );

    setIcon( iconNoDaemon );
    QTimer::singleShot( 500, this, SLOT( firstRun() ) );
    loadModules();
}

Applet::~Applet()
{
    unloadModules();
}

void Applet::showNoDaemonMessage()
{
    showMessage( NoDaemon );
}

void Applet::showHideMonitor()
{
    if ( m_monitor->isHidden() )
    {
        m_actionHide->setText( tr( "&Hide" ) );
        m_actionHide->setIcon( iconHide );
        m_monitor->show();
    }
    else
    {
        m_actionHide->setText( tr( "&Show" ) );
        m_actionHide->setIcon( iconShow );
        m_monitor->hide();
    }
}

void Applet::clicked( QSystemTrayIcon::ActivationReason reason )
{
    if ( reason == QSystemTrayIcon::Trigger )
        showHideMonitor();
}

void Applet::showMessage( Message msg )
{
    QString title;
    QString text;

    switch ( msg )
    {
    case Connecting:
        title = tr( "Establishing connection" );
        text  = tr( "Connection is being established..." );
        break;
    case ConnectionTimeout:
        title = tr( "Connection timeout" );
        text  = tr( "Could not establish connection - provider does not respond." );
        break;
    case RestartingModem:
        title = tr( "Restarting modem" );
        text  = tr( "Modem is being restarted.\nThis may take few seconds..." );
        break;
    case ConnectionTerminated:
        title = tr( "Connection terminated" );
        text  = tr( "Connection has been terminated..." );
        break;
    case NoDaemon:
        title = tr( "Daemon error" );
        text  = tr( "Probably the UbuDSL Daemon is not running. Please "
                    "run it if you want to use this applet." );
        break;
    case IncorrectDNS:
        title = tr( "Incorrect DNS servers" );
        text  = tr( "UbuDSL has detected damaged DNS servers "
                    "configuration. Trying to set OpenDNS as "
                    "the default DNS servers..." );
        break;
    case OpenDNS_Success:
        title = tr( "OpenDNS set" );
        text  = tr( "UbuDSL has successfully set OpenDNS "
                    "as the default DNS servers..." );
        break;
    case OpenDNS_Failed:
        title = tr( "Setting OpenDNS failed" );
        text  = tr( "UbuDSL could not set OpenDNS as the "
                    "default DNS servers..." );
        break;
    case Synchronising:
        title = tr( "Waiting for synchronisation" );
        text  = tr( "Please wait while modem is now "
                    "synchronising..." );
        break;
    case SynchronisationFailed:
        title = tr( "Synchronisation failed" );
        text  = tr( "Connection could not be established "
                    "because modem failed to synchronise..." );
        break;
    case CompilingDriver:
        title = tr( "Compiling driver" );
        text  = tr( "Please wait while the modem's driver is "
                    "being compiled..." );
        break;
    case CompilationFailed:
        title = tr( "Compilation failed!" );
        text  = tr( "Compilation has failed and most probably you will not "
                    "be able to establish connection. Please generate and send "
                    "us the log - we will try to help you out." );
        break;
    case CompilationSucceed:
        title = tr( "Compilation succeed" );
        text  = tr( "Compilation has been finished successfully. "
                    "You should now be able to establish connection..." );
        break;
    case RestartingModemOnHangUp:
        title = tr( "Modem hangup" );
        text  = tr( "Modem is being restarted because of its hangup. After the "
                    "restart UbuDSL will try again to establish connection." );
        break;
    case FirstRun:
        title = tr( "First run" );
        text  = tr( "It seems that you have run the Applet first time. "
                    "This is an open source software and you may help "
                    "improving it by your support. You can find out how "
                    "to do this by opening the About dialog." );
        break;
    }

    if ( !title.isEmpty() && !text.isEmpty() )
        showMessage( title, text );
}

void Applet::enableGenerateLog()
{
    m_actionGenerateLog->setEnabled( true );
}

void Applet::disableGenerateLog()
{
    m_actionGenerateLog->setEnabled( false );
}

void Applet::establishingConnection()
{
    m_connectingTimer->start( 500 );
    setIcon( iconConnecting );
    showMessage( Connecting );
    setToolTip( tr( "Establishing connection..." ) );
}

void Applet::connectionEstablished()
{
    QString dnsServer = tr( "error" );
    m_connectingTimer->stop();
    m_actionConnect->setIcon( iconDisconnect );

    setIcon( iconConnected );

    m_actionConnect->setText( tr( "&Disconnect" ) );
    m_monitor->setConnectButtonText( m_actionConnect->text().toLower() );

    QDBusReply<QString> interface = daemon_iface.call( "getInterface" );
    QDBusReply<QString> ip = daemon_iface.call( "getIP" );

    QDBusReply<QStringList> dns = daemon_iface.call( "getDNSServers" );
    if ( dns.isValid() )
    {
        if ( dns.value().count() < 1 )
        {
            dnsServer = tr( "none" );
        }
        else
        {
            dnsServer = dns.value().at( 0 );
        }
    }

    if ( interface.isValid() && ip.isValid() )
    {
        QString strInterface  = tr( "Interface:" );
        QString strIPAddress  = tr( "IP Address:" );
        QString strPrimaryDNS = tr( "Primary DNS:" );

        QString title = tr( "Connection established" );
        QString text  = "<table><tr><td>%1 </td><td>%2</td></tr>"
                        "<tr><td>%3 </td><td>%4</td></tr>"
                        "<tr><td>%5 </td><td>%6</td></tr></table>";

        QString tip = title + "\n"  + strInterface  + " " + interface.value() + "\n"
                      + strIPAddress  + " " + ip.value()        + "\n"
                      + strPrimaryDNS + " " + dnsServer         + ".";

        text = text.arg( strInterface, interface.value(), strIPAddress, ip.value(), strPrimaryDNS, dnsServer );

        showMessage( title, text );
        setToolTip( tip );
    }

    emit playSndConnected();
}

void Applet::connectionTerminated( bool message )
{
    m_connectingTimer->stop();
    m_actionConnect->setIcon( iconConnect );

    setIcon( iconDisconnected );

    m_actionConnect->setText( tr( "&Connect" ) );
    m_monitor->setConnectButtonText( m_actionConnect->text().toLower() );
    setToolTip( tr( "Disconnected" ) );

    if ( message )
        showMessage( ConnectionTerminated );

    emit playSndDisconnected();
}

void Applet::restartModem()
{
    daemon_iface.call( "restartModem" );
}

void Applet::checkState( bool firstTime )
{
    QDBusReply<int> state = daemon_iface.call( "getConnectionState" );

    if ( state.isValid() )
    {
        QDBusReply<bool> modemPlugged = daemon_iface.call( "getIsModemPlugged" );

        m_actionConnect->setEnabled( modemPlugged.isValid() && modemPlugged.value() );
        m_actionReconnect->setEnabled( modemPlugged.isValid() && modemPlugged.value() );
        m_monitor->setConnectEnabled( modemPlugged.isValid() && modemPlugged.value() );

        if ( modemPlugged.isValid() && !modemPlugged.value() )
        {
            setIcon( iconNoModem );
            setToolTip( tr( "Modem is unplugged..." ) );

            QString title = tr( "Modem unplugged" );
            QString text  = tr( "Modem is unplugged..." );

            showMessage( title, text );
            emit playSndUnplugged();
        }
        else if ( state == ConnectionState::Connected )
            connectionEstablished();
        else if ( state == ConnectionState::Connecting )
            establishingConnection();
        else if ( state == ConnectionState::Disconnected )
            connectionTerminated( firstTime );
    }
    else if ( !daemon_iface.isValid() )
    {
        showMessage( NoDaemon );

        emit playSndError();

        m_actionConnect->setEnabled( false );
        m_actionReconnect->setEnabled( false );
        m_monitor->setConnectEnabled( false );
        setToolTip( tr( "Failed to connect to the daemon!" ) );
    }
}

void Applet::showSettings()
{
    if ( m_settingsDialog->isVisible() )
        return;

    if ( !m_settings.value( "show_password", true ).toBool() )
        m_settingsDialog->ui.lePassword->setEchoMode( QLineEdit::Password );

    m_settingsDialog->ui.buttonSave->setDisabled( true );

    QDBusReply<QString> strReply;
    QDBusReply<bool>    boolReply;
    QString             dataError( tr( "data error" ) );

    strReply = daemon_iface.call( "getService" );
    if ( !strReply.isValid() || strReply.value() == "error" )
        m_settingsDialog->ui.labelName->setText( dataError );
    else
        m_settingsDialog->ui.labelName->setText( strReply.value() );

    strReply = daemon_iface.call( "getProtocol" );
    if ( !strReply.isValid() || strReply.value() == "error" )
        m_settingsDialog->ui.labelProtocol->setText( dataError );
    else
        m_settingsDialog->ui.labelProtocol->setText( strReply.value() );

    strReply = daemon_iface.call( "getVPI" );
    if ( !strReply.isValid() || strReply.value() == "error" )
        m_settingsDialog->ui.labelVPI->setText( dataError );
    else
        m_settingsDialog->ui.labelVPI->setText( strReply.value() );

    strReply = daemon_iface.call( "getVCI" );
    if ( !strReply.isValid() || strReply.value() == "error" )
        m_settingsDialog->ui.labelVCI->setText( dataError );
    else
        m_settingsDialog->ui.labelVCI->setText( strReply.value() );

    boolReply = daemon_iface.call( "getConnectAtBoot" );
    m_settingsDialog->ui.cbConnectAtBoot->setEnabled( boolReply.isValid() );
    if ( boolReply.isValid() )
        m_settingsDialog->ui.cbConnectAtBoot->setChecked( boolReply.value() );

    boolReply = daemon_iface.call( "getConnectAtModemPlug" );
    m_settingsDialog->ui.cbConnectAtModemPlug->setEnabled( boolReply.isValid() );
    if ( boolReply.isValid() )
        m_settingsDialog->ui.cbConnectAtModemPlug->setChecked( boolReply.value() );


    QDBusReply<QStringList> listReply = daemon_iface.call( "getLoginAndPassword" );
    if ( !listReply.isValid() || listReply.value().count() != 2 )
    {
        m_settingsDialog->ui.leLogin->setText( dataError );
        m_settingsDialog->ui.lePassword->setText( dataError );
        m_settingsDialog->ui.leLogin->setEnabled( false );
        m_settingsDialog->ui.lePassword->setEnabled( false );
    }
    else
    {
        m_settingsDialog->ui.leLogin->setText( listReply.value().at( 0 ) );
        m_settingsDialog->ui.lePassword->setText( listReply.value().at( 1 ) );
        m_settingsDialog->ui.leLogin->setEnabled( true );
        m_settingsDialog->ui.lePassword->setEnabled( true );
    }

    listReply = daemon_iface.call( "getDNSServers" );
    if ( !listReply.isValid() )
    {
        m_settingsDialog->ui.labelDNS1->setText( dataError );
        m_settingsDialog->ui.labelDNS2->setText( dataError );
    }
    else
    {
        if ( listReply.value().count() == 0 )
        {
            m_settingsDialog->ui.labelDNS1->setText( tr( "none" ) );
            m_settingsDialog->ui.labelDNS2->setText( tr( "none" ) );
        }
        else if ( listReply.value().count() == 1 )
        {
            m_settingsDialog->ui.labelDNS1->setText( listReply.value().at( 0 ) );
            m_settingsDialog->ui.labelDNS2->setText( "" );
        }
        else
        {
            m_settingsDialog->ui.labelDNS1->setText( listReply.value().at( 0 ) );
            m_settingsDialog->ui.labelDNS2->setText( listReply.value().at( 1 ) );
        }
    }


    QDBusReply<int> intReply = daemon_iface.call( "getIdlenessTime" );
    m_settingsDialog->ui.cbIdle->setEnabled( intReply.isValid() );
    if ( intReply.isValid() )
    {
        if ( intReply.value() > 0 && intReply.value() <= 360 )
        {
            m_settingsDialog->ui.cbIdle->setChecked( true );
            m_settingsDialog->ui.spinIdle->setValue( intReply.value() );
        }
        else
        {
            m_settingsDialog->ui.cbIdle->setChecked( false );
        }
    }

    m_settingsDialog->exec();
}

void Applet::applySettings()
{
    if ( m_settingsDialog->getAccountDataChanged() )
    {
        daemon_iface.call( "setAccountData", m_settingsDialog->ui.leLogin->text(), m_settingsDialog->ui.lePassword->text() );
    }

    if ( m_settingsDialog->getConnectAtBootChanged() )
    {
        daemon_iface.call( "setConnectAtBoot", m_settingsDialog->ui.cbConnectAtBoot->isChecked() );
    }

    if ( m_settingsDialog->getConnectAtModemPlugChanged() )
    {
        daemon_iface.call( "setConnectAtModemPlug", m_settingsDialog->ui.cbConnectAtModemPlug->isChecked() );
    }

    if ( m_settingsDialog->getIdleTimeChanged() )
    {
        daemon_iface.call( "setIdlenessTime", m_settingsDialog->ui.spinIdle->value() );
    }

    if ( m_settingsDialog->getIdleChanged() && !m_settingsDialog->ui.cbIdle->isChecked() )
    {
        daemon_iface.call( "setIdlenessTime", 0 );
    }

    m_settingsDialog->getData();
}

void Applet::modemUnplugged()
{
    setIcon( iconNoModem );
    setToolTip( "Modem is unplugged..." );

    m_actionConnect->setEnabled( false );
    m_actionReconnect->setEnabled( false );
    m_monitor->setConnectEnabled( false );

    QString title = tr( "Modem unplugged" );
    QString text  = tr( "Modem has been unplugged..." );

    showMessage( title, text );

    emit playSndUnplugged();
}

void Applet::newModemPlugged( const QString &name )
{
    QString title = tr( "New modem plugged in" );
    QString text  = tr( "%1 has been plugged in." );
    QString textL = tr( "%1 has been plugged in. Do you want to set it as the default modem?" );

    showMessage( title, text.arg( name ) );

    emit playSndPlugged();

    m_newModemMsgBox->setWindowTitle( title );
    m_newModemMsgBox->setText( textL.arg( name ) );
    m_newModemMsgBox->exec();

    if ( m_newModemMsgBox->clickedButton() == m_btnYes )
    {
        daemon_iface.call( "useNewModem" );
    }
}

void Applet::modemPlugged( const QString &name )
{
    QString title = tr( "Modem plugged" );
    QString text  = ( name.isEmpty() ? tr( "Modem has been plugged in..." ) : tr( "%1 has been plugged in..." ) );

    showMessage( title, text.arg( name ) );
    m_actionConnect->setEnabled( true );//TODO: remove after synchro detection is implemented
    m_actionReconnect->setEnabled( true );
    m_monitor->setConnectEnabled( true );
    setToolTip( tr( "Disconnected" ) );
    setIcon( iconDisconnected );

    emit playSndPlugged();
}


void Applet::incorrectDNS()
{
    showMessage( IncorrectDNS );
}

void Applet::settingOpenDNS( const bool success )
{
    if ( success )
        showMessage( OpenDNS_Success );
    else
        showMessage( OpenDNS_Failed );
}

void Applet::newModemUnplugged()
{
    QString title = tr( "New modem unplugged" );
    QString text  = tr( "New modem has been unplugged..." );

    showMessage( title, text );

    m_newModemMsgBox->reject();

    emit playSndUnplugged();
}

void Applet::accountDataChanged( const bool success )
{
    QString title = ( success ? tr( "Account data changed" ) : tr( "Failed to change data" ) );
    QString text;

    if ( success )
    {
        text = tr( "Account data changed successfully..." );
    }
    else
    {
        text = tr( "An error has occured while changing the account data..." );
    }

    showMessage( title, text );
}

void Applet::checkDaemonConnection()
{
    QDBusReply<bool> reply = daemon_iface.call( "reply" );

    if ( m_daemonOk == reply.isValid() )
        return;

    if ( reply.isValid() )
    {
        checkState( false );
    }
    else
    {
        setIcon( iconNoDaemon );

        m_monitor->daemonError();

        QString title = tr( "Daemon connection lost" );
        QString text  = tr( "The applet has lost the connection "
                            "with UbuDSL daemon. Probably the daemon "
                            "had been terminated." );
        showMessage( title, text );
        emit playSndError();

        setToolTip( tr( "Daemon connection lost" ) );
    }

    m_daemonOk = reply.isValid();
}

void Applet::connecting()
{
    static bool a = true;

    if ( a )
        setIcon( iconConnecting );
    else
        setIcon( iconConnected );

    a = !a;
}

void Applet::invalidAccountData()
{
    static bool slotFinished( true );

    if ( !slotFinished )
        return;

    slotFinished = false;

    m_connectingTimer->stop();
    m_actionConnect->setIcon( iconConnect );

    setIcon( iconDisconnected );

    QString title = tr( "Invalid login and/or password!" );
    QString text  = tr( "You have entered an invalid login and/or password "
                        "and the connection could not be established. Please "
                        "open the Settings dialog (right click on the Applet's "
                        "icon and choose the right position from the menu, or, "
                        "click the Open Settings button below) and "
                        "change your service's login and password to correct ones. "
                        "After you did this, you can try to connect again." );

    QMessageBox msgBox( QMessageBox::Critical, title, text, QMessageBox::Ok );
    msgBox.setAttribute( Qt::WA_QuitOnClose, false );

    QAbstractButton *settingsButton = msgBox.addButton( "&Open Settings", QMessageBox::ActionRole );

    msgBox.exec();

    if ( msgBox.clickedButton() == settingsButton )
        emit showSettings();

    slotFinished = true;
}

void Applet::compilingDriver()
{
    showMessage( CompilingDriver );

    m_actionConnect->setEnabled( false );
    m_actionReconnect->setEnabled( false );
    m_monitor->setConnectEnabled( false );
}

void Applet::compilationFinished( const bool success )
{
    m_actionConnect->setEnabled( true );
    m_actionReconnect->setEnabled( true );
    m_monitor->setConnectEnabled( true );

    if ( success )
        showMessage( CompilationSucceed );
    else
        showMessage( CompilationFailed );
}

void Applet::PADOTimeout()
{
    static bool slotFinished( true );

    if ( !slotFinished )
        return;

    slotFinished = false;
    m_connectingTimer->stop();
    m_actionConnect->setIcon( iconConnect );

    setIcon( iconDisconnected );

    QString title = tr( "Timeout waiting PADO packets!" );
    QString text  = tr( "Can not establish connection. PPP daemon's message:\n\t"
                        "Timeout waiting PADO packets." );

    QMessageBox msgBox( QMessageBox::Critical, title, text, QMessageBox::Ok );
    msgBox.setAttribute( Qt::WA_QuitOnClose, false );

    msgBox.exec();
    slotFinished = true;
}

void Applet::modemStillHangsUp()
{
    static bool slotFinished( true );

    if ( !slotFinished )
        return;

    slotFinished = false;

    m_connectingTimer->stop();
    m_actionConnect->setIcon( iconConnect );

    setIcon( iconDisconnected );

    QString title = tr( "Modem hangup!" );
    QString text  = tr( "Can not establish connection. The modem had already been restarted "
                        "but it still hangs up..." );

    QMessageBox msgBox( QMessageBox::Critical, title, text, QMessageBox::Ok );
    msgBox.setAttribute( Qt::WA_QuitOnClose, false );

    msgBox.exec();
    slotFinished = true;
}

void Applet::loadModules()
{
    qDebug() << "loadModules()";
    typedef void* ( *InitFunction )( Applet * );

    QDir modulesDir( UBUDSL_PREFIX + "/lib/ubudsl/modules" );
    QRegExp moduleRegExp( "^libubudsl_module_.{1,30}\\.so$" );

    if ( !modulesDir.exists() )
    {
        modulesDir.setPath( UBUDSL_PREFIX + "/lib64/ubudsl/modules" );
        if ( !modulesDir.exists() ) return;
    }

    QFileInfoList files = modulesDir.entryInfoList();
    QListIterator<QFileInfo> it( files );

    while ( it.hasNext() )
    {
        QFileInfo fi( it.next() );

        if ( !fi.isFile() || !moduleRegExp.exactMatch( fi.fileName() ) )
            continue;

        QLibrary *module = new QLibrary( fi.absoluteFilePath() );
        if ( !module->load() )
        {
            qDebug() << "Applet::loadModules(): failed to load module:" << fi.absoluteFilePath();
            qDebug() << module->errorString();

            delete module;
            continue;
        }

        InitFunction init = ( InitFunction ) module->resolve( "module_init" );
        if ( init )
        {
            qDebug() << "Applet::loadModules():" << fi.absoluteFilePath() << "module_init symbol resolved successfully.";
            init( this );
        }

        m_modules.append( module );
    }
}

void Applet::unloadModules()
{
    qDebug() << "unloadModules()";

    typedef void* ( *ExitFunction )( void );
    ModulesListIterator it( m_modules );

    while ( it.hasNext() )
    {
        QLibrary *module = it.next();

        if ( module == NULL )
            continue;

        if ( !module->isLoaded() )
        {
            delete module;
            continue;
        }

        ExitFunction exit = ( ExitFunction ) module->resolve( "module_exit" );

        if ( exit )
        {
            qDebug() << "Calling module_exit function.";
            exit();
        }

        delete module;
    }
}

void Applet::synchronisationFailed()
{
    connectionTerminated( false );
    showMessage( SynchronisationFailed );
}

void Applet::restartModemStarted()
{
    connectionTerminated( false );
    showMessage( RestartingModem );
    m_actionRestartModem->setEnabled( false );
}

void Applet::connectionTimeout()
{
    connectionTerminated( false );
    showMessage( ConnectionTimeout );
}

void Applet::setConnectionState( const int &state )
{
    if ( m_connectingTimer->isActive() && state == ConnectionState::Disconnected )
    {
        connectionTerminated( false );
    }
}

bool Applet::getIsInAutostart()
{
    QDir d( autostartPath );

    if ( !d.exists() )
        return false;

    return QFile::exists( autostartPath + "/" + desktopFileName );
}

bool Applet::addToAutostart()
{
    QDir d( autostartPath );
    if ( !d.exists() )
    {
        if ( !d.mkpath( autostartPath ) )
        {
            m_settings.setValue( "autostart", false );
            return false;
        }
    }

    bool ret;
    QFile file( autostartPath + "/" + desktopFileName );

    if ( file.open( QFile::WriteOnly ) )
    {
        ret = true;

        file.write( "[Desktop Entry]\n" );
        file.write( "Version=1.0\n" );
        file.write( "Encoding=UTF-8\n" );
        file.write( "Name=UbuDSL Applet\n" );
        file.write( "Type=Application\n" );
        file.write( "Terminal=false\n" );
        file.write( "Icon=ubudsl.png\n" );
        file.write( "Exec=UbuDSL_Applet " );
        file.write( "--delay=" );
        file.write( qPrintable( m_settings.value( "applet_delay", 10 ).toString() ) );
        file.write( "\n" );
        file.write( "X-GNOME-Autostart-enabled=true\n" );

        file.close();
    }
    else
    {
        ret = false;
    }

    m_settings.setValue( "autostart", ret );
    return ret;
}

bool Applet::removeFromAutostart()
{
    QDir d( autostartPath );
    if ( !d.exists() || !QFile::exists( autostartPath + "/" + desktopFileName ) )
    {
        m_settings.setValue( "autostart", false );
        return true;
    }

    bool ret = QFile::remove( autostartPath + "/" + desktopFileName );

    m_settings.setValue( "autostart", false );
    return ret;
}

void Applet::autostartActionTriggered()
{
    if ( m_actionAutostart->isChecked() )
        addToAutostart();
    else
        removeFromAutostart();
}

void Applet::firstRun()
{
    m_timer->start();
    checkState();

    if ( m_settings.value( "first_run", true ).toBool() || m_settings.value( "applet_version", 0 ).toInt() < MIN_APPLET_VERSION )
    {
        removeFromAutostart(); // Just in case of an old .desktop file.

        FirstRunWizard wizard( &m_settings );
        wizard.exec();

        m_settings.setValue( "first_run", false );
    }

    m_settings.setValue( "applet_version", APPLET_VERSION );

    if ( m_settings.value( "autostart", false ).toBool() && !getIsInAutostart() )
        addToAutostart();
    else if ( !m_settings.value( "autostart", false ).toBool() && getIsInAutostart() )
        removeFromAutostart();

    QDBusReply<bool> reply = daemon_iface.call( "getIsConfigurationOk" );

    if ( reply.isValid() && !reply.value() )
    {
        QMessageBox msgBox;

        msgBox.setAttribute( Qt::WA_QuitOnClose, false );
        msgBox.setIcon( QMessageBox::Critical );

        QAbstractButton *btnReject = msgBox.addButton( tr( "&Cancel" ), QMessageBox::RejectRole );
        QAbstractButton *btnAccept = msgBox.addButton( tr( "&Run configuration" ), QMessageBox::AcceptRole );

        Q_UNUSED( btnReject );

        msgBox.setWindowTitle( tr( "Configuration error!" ) );

        msgBox.setText( tr( "A wrong configuration has been detected. "
                            "This means that you haven't configured your "
                            "connection yet or the configuration is too old "
                            "(due to the important changes in the UbuDSL "
                            "code you have to reconfigure the connection.\n\n"
                            "To start the configuration you can click the \"%1\" "
                            "button. You can also do this by yourself trough "
                            "the system configuration menu)." ).arg( btnAccept->text().remove( "&" ) ) );

        msgBox.exec();

        if ( msgBox.clickedButton() == btnAccept )
            QProcess::startDetached( "ubudsl" );
    }

    if ( m_settings.value( "show_help_dialog", true ).toBool() )
    {
        m_helpDialog->exec();

        m_settings.setValue( "show_help_dialog", !m_helpDialog->getDontShowAgain() );
    }
}

void Applet::showMessage( const QString &title, const QString &message )
{
    QDBusMessage m = QDBusMessage::createMethodCall( "org.kde.VisualNotifications", "/VisualNotifications", "org.kde.VisualNotifications", "Notify" );

    QList<QVariant> args;
    args.append("UbuDSL");
    args.append((uint)0);
    args.append(QString());
    args.append("/usr/share/pixmaps/ubudsl.png");
    args.append(title);
    args.append(message);
    args.append(QStringList());
    args.append(QVariantMap());
    args.append(10000);

    m.setArguments(args);

    QDBusMessage replyMsg = QDBusConnection::sessionBus().call(m);

    if(replyMsg.type() != QDBusMessage::ReplyMessage)
    {
        m_bubble->showMessage( geometry(), title, message );
    }
}

/*
void Applet::addModuleAction( QAction *action )
{
	if ( m_modulesActions.count() == 0 )
	    m_menu->insertAction( m_separator3, m_separator2 );
	    
	foreach ( QAction* menuAction, m_menu->actions() )
	{
		if ( menuAction == action )
		    return;
	}
	
	m_modulesActions << action;
	m_menu->insertAction( m_separator3, action );
}

void Applet::removeModuleAction( QAction *action )
{
	m_menu->removeAction( action );
	m_modulesActions.removeAll( action );
	
	if ( m_modulesActions.count() == 0 )
		m_menu->removeAction( m_separator2 );
}*/

