/*
 * File: Compiler.h
 * Author: Adrian Łubik (2008)
 * License: GNU LGPL v3
 */

#ifndef SCRIPTWRITER_H
#define SCRIPTWRITER_H

#include <QString>
#include <QStringList>
#include <QDebug>
#include <QSettings>
#include <QFile>
#include <QDir>
#include <QFileInfo>
#include <QThread>

#include "MyProcess.h"
#include "SystemInfo.h"
#include "FileManager.h"
#include "ModemManager.h"

/**
  * A class for managing drivers' sources.
  */
class Compiler: public QObject
{
    Q_OBJECT
public:
    /**
      * Class constructor.
      * @param driver is a ModemDriver struct instance.
      * @see ModemManager
      */
    Compiler( const ModemManager::ModemDriver driver, QThread *parent = 0 );

    /**
      * Class destructor.
      */
    virtual ~Compiler();

    /**
      * Compiles the driver specified by the ModemDriver stuct given in the constructor.
      * @return Compilation process's exit code.
      */
    int  compileDriver();

    /**
      * Extracts the driver.
      */
    void extractDriver();

    /**
      * Installs the driver.
      * This method does not use 'make' to install the driver.
      * Instead of this, compiled driver is being found and copied to
      * /lib/modules/_CURRENT_KERNEL_/extra.
      */
    int  installDriver();

    /**
      * Backs up the driver to /tmp/ubudsl_drv_backup/_KERNEL_VERSION_
      */
    void backupDriver();

    /**
      * Restores backed up driver(s).
      */
    void restoreDriver();

    /**
      * If a patch named "DriverName-DriverVersion_KernelVersion.patch"-like is found
      * in the bundled driver's dir, the driver is being automatically patched.
      */
    void patchDriver();

private slots:
    void debugDriverCompileProcess( QByteArray ba );
    void debugDriverRemoveProcess( QByteArray ba );
    void debugDriverInstallProcess( QByteArray ba );

private:
    MyProcess   *m_compileDriverProcess;
    MyProcess   *m_installDriverProcess;
    MyProcess   *m_extractDriverProcess;
    QProcess    *m_patchProcess;

    QString      extractedDriverPath;
    QString      driverName;
    QString      driverVersion;
    QString      extractionPath;
    QString      driverBundlePath;
    QString      compiledDriverName;
    QStringList  compilationArguments;
    QStringList  installationArguments;
    QStringList  environmentVariables;
    QByteArray   compilerDebug; //grabbed messages of make process
    QStringList  backupList;    //list of drivers to restore if compilation failed
    QString      kernel;
    QString      backupPath;
    QDir        *backupDir;
    QStringList  backedUpModules;
};

#endif

