#include "Stats.h"

Stats::Stats( const QString &interface )
: m_interface( interface ),
  m_timer( new QTimer( this ) ),
  m_ppp_s( -1 )
{
    connect( m_timer, SIGNAL(timeout()), this, SLOT(update()) );
}

void Stats::setInterface( const QString &interface )
{
    m_interface = interface;
}

/*------------------------------------------------------------------------*
 *  GET_STATISTICS - Gets RX/TX Packets, Bytes and Errors.  Expects no    *
 *  arguments and returns 0 on success, -1 on failure.  Places statistics *
 *  in the structure 'stats'.  Sets the global 'ppp_opened'.              *
 *------------------------------------------------------------------------*/
int Stats::getStat()
{
    struct ppp_stats ppp_cur;
    static int ppp_opened = 0;
    if (!ppp_opened)
    {   /* If not opened yet... Open the PPP device */
        memset (&ppp_cur, 0, sizeof (ppp_cur));
        if ((m_ppp_s = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
	{
	  return -1;
	}
        ppp_opened = 1;
    }
    if (getPPPStats (&ppp_cur) < 0)
    {
      return -1;
    }

    /* Update the information */
    m_stats.tx_packets = ppp_cur.p.ppp_opackets;
    m_stats.rx_packets = ppp_cur.p.ppp_ipackets;
    m_stats.rx_bytes = ppp_cur.p.ppp_ibytes;
    m_stats.tx_bytes = ppp_cur.p.ppp_obytes;
    m_stats.rx_errors = ppp_cur.p.ppp_ierrors;
    m_stats.tx_errors = ppp_cur.p.ppp_oerrors;
    return 0;
}

/*---------------------------------------------------------------------------*
 * GET_PPP_STATS - Gets PPP stats struct and points the arg  to it.  Expects *
 * a pointer to a ppp_stats struct.  Returns 0 on success, -1 on failure.    *
 *---------------------------------------------------------------------------*/
int Stats::getPPPStats( struct ppp_stats *cur )
{

    struct ifpppstatsreq req;
    memset (&req, 0, sizeof (req));
    req.stats_ptr = (char *) &req.stats;

    strncpy (req.ifr__name, m_interface.toLatin1().constData(), sizeof(req.ifr__name) - 1);
    if (ioctl (m_ppp_s, SIOCGPPPSTATS, &req) < 0)
    {
        return -1;
    }

    *cur = req.stats;
    return 0;
}

void Stats::startStatsTimer()
{
    if ( !m_timer->isActive() )
        m_timer->start( 1000 );
}

void Stats::stopStatsTimer()
{
    if ( m_timer->isActive() )
    {
        m_timer->stop();

        if ( m_stats.rx_bytes && m_stats.tx_bytes )
            emit statsError( m_stats.rx_bytes, m_stats.tx_bytes );
    }
}

void Stats::update()
{
    if ( getStat() != 0 )
    {
        if ( m_stats.rx_bytes && m_stats.tx_bytes )
            emit statsError( m_stats.rx_bytes, m_stats.tx_bytes );

        return;
    }

    static double old_tx_bytes = m_stats.tx_bytes;
    static double old_rx_bytes = m_stats.rx_bytes;

    m_stats.tx_bps = m_stats.tx_bytes - old_tx_bytes;
    m_stats.rx_bps = m_stats.rx_bytes - old_rx_bytes;

    old_tx_bytes = m_stats.tx_bytes;
    old_rx_bytes = m_stats.rx_bytes;

    emit statsAvailable( m_stats );
}

