#ifndef DAEMON_H_
#define DAEMON_H_

#include <QtCore>
#include <QtDBus>
#include <QtNetwork>

#include <sys/klog.h>
#include <signal.h>

#include "UbuDSL_Message.h"
#include "ConnectThread.h"
#include "DisconnectThread.h"
#include "RestartModem.h"
#include "CompilerThread.h"
#include "Stats.h"
#include "Globals.h"
#include "libubudsl/libubudsl.h"


class UbuDSL_Daemon : public QObject
{
    Q_OBJECT
    Q_CLASSINFO( "D-Bus Interface", "com.ubudsl.Daemon" )

public:
    UbuDSL_Daemon( QObject* parent = 0 );
    virtual ~UbuDSL_Daemon();
    QDBusConnection dbus;

    static int  getModemState();
    static void debugHandler( QtMsgType type, const char *msg );
public slots:
//const:
    const QString getIP( QString interface = "" );
    const QString getDefaultRoute();

//void:
    void    setConnectAtBoot( bool connect );
    void    setConnectAtModemPlug( bool connect );
    void    establishConnection();
    void    terminateConnection( bool message = true );
    void    restartModem();
    void    setIdlenessTime( const int &idleness );
    void    useNewModem();
    void    setAccountData( const QString &login, const QString &password );
    void    update();

//bool:
    bool    getIsModemPlugged();
    bool    getIsCompiling();
    bool    getIsConfigurationOk();
    bool    getConnectAtBoot()      { return m_settings->value( "connect_at_boot", false ).toBool();       }
    bool    getConnectAtModemPlug() { return m_settings->value( "connect_at_modem_plug", false ).toBool(); }
    bool    reply()                 { return true;    /*called by applet to check connection with daemon*/ }

//QString:
    QString getVPI()                { return m_settings->value( "vpi", "error" ).toString();      }
    QString getVCI()                { return m_settings->value( "vci", "error" ).toString();      }
    QString getProtocol()           { return m_settings->value( "protocol", "error" ).toString(); }
    QString getService()            { return m_settings->value( "service", "error" ).toString();  }

    QStringList getDNSServers()     { return dns.getDNSServers(); }

    static QString getInterface();

    int getConnectionState();
    int getIdlenessTime(); //From settings

    QStringList getLoginAndPassword();

private slots:
    void connectingStarted();
    void connectingFinished();
    void deviceAdded( const QString udi );
    void deviceRemoved( const QString udi );
    void verifyDevice();
    void verifyDNS();
    void catchTransmissionStatistics( const Stats::DataStats &stats );
    QTime getConnectionTime();
    void debugConnecting( QByteArray msg );
    void compilationFinished();
    void compilationStarted();
    void saveStats( const double &lastRxBytes, const double &lastTxBytes );
    void _restartModemStarted();
    void _restartModemFinished();

private:
    void writeConnectionLog( const QString &str );
    void wait( int secs );
    void initModemsMap();
    void setOpenDNS();
    void initDaemon();
    void testDriver();
    static void refreshKernelBuffer();


    struct Modem
    {
        QString name;
        QString driver;
        QMultiMap<int, int> usb_id;
    };

    bool m_justRun;
    bool m_connecting;
    bool m_modemIsPlugged;
    bool m_compiling;

    ConnectThread    *m_connectThread;
    DisconnectThread *m_disconnectThread;

    int m_newVendorID;
    int m_newProductID;

    QDBusInterface *m_halManager;

    static QFile *m_logFile;

    QList<Modem> m_modems;

    static QMutex m_mutexLog;
    QMutex m_connectMutex;
    QMutex m_disconnectMutex;

    QRegExp m_connectedRegExp;
    QRegExp m_connectingRegExp;
    QRegExp m_invalidDataRegExp;
    QRegExp m_unrecognizedOptionRegExp;
    static QSettings *m_settings;

    QString m_IP;
    QString m_lastUDI;
    QString m_newModemName;
    QString m_newUDI;
    static QString m_kernelBuffer;

    QTime   m_connectionTime;

    QTimer *m_deviceTimer;
    Stats *m_stats;

    unsigned long m_rx_bytes;
    unsigned long m_tx_bytes;

    DNS_Manager     dns;
    RestartModem   *m_restartModem;
    CompilerThread *m_compilerThread;

    friend class ConnectThread;


signals:
    Q_SCRIPTABLE void transmissionSpeed( const unsigned int &rx, const unsigned int &tx );
    Q_SCRIPTABLE void transmissionErrors( const unsigned int &rx, const unsigned int &tx );
    Q_SCRIPTABLE void transmissionBytes( const double &rx, const double &tx );
    Q_SCRIPTABLE void IP( const QString &ip );
    Q_SCRIPTABLE void defaultRoute( const QString &iface );
    Q_SCRIPTABLE void connectionTime( const int &h, const int &m, const int &s );

    Q_SCRIPTABLE void modemPlugged( const QString &name );
    Q_SCRIPTABLE void newModemPlugged( const QString &name );
    Q_SCRIPTABLE void modemUnplugged();
    Q_SCRIPTABLE void newModemUnplugged();
    Q_SCRIPTABLE void incorrectDNS();
    Q_SCRIPTABLE void settingOpenDNS( const bool success );

    Q_SCRIPTABLE void connectionEstablished();
    Q_SCRIPTABLE void connectionTerminated( const bool showMessage );
    Q_SCRIPTABLE void establishingConnection();
    Q_SCRIPTABLE void setEnabledConnect( const bool enable );
    Q_SCRIPTABLE void connectionState( const int &state );
    Q_SCRIPTABLE void connectionTimeout();
    Q_SCRIPTABLE void restartModemStarted();
    Q_SCRIPTABLE void restartModemFinished();
    Q_SCRIPTABLE void accountDataChanged( const bool success );

    Q_SCRIPTABLE void compilingDriver();
    Q_SCRIPTABLE void compilationFinished( const bool success );
    Q_SCRIPTABLE void synchronising();
    Q_SCRIPTABLE void synchronisationFailed();
    Q_SCRIPTABLE void invalidAccountData();
    Q_SCRIPTABLE void modemStillHangsUp();
    Q_SCRIPTABLE void restartOnModemHangUp();
    Q_SCRIPTABLE void PADOTimeout();
};

#endif /*DAEMON_H_*/
